/* $Id: mga_common_mmio.c,v 1.5 1998/11/19 02:43:03 marcus Exp $
********************************************************************************
**	MGA I/O routines
********************************************************************************
**
   Copyright (C)	1997		Jonas Borgstrom
**
   Permission is hereby granted, free of charge, to any person obtaining a
   copy of this software and associated documentation files (the "Software"),
   to deal in the Software without restriction, including without limitation
   the rights to use, copy, modify, merge, publish, distribute, sublicense,
   and/or sell copies of the Software, and to permit persons to whom the
   Software is furnished to do so, subject to the following conditions:

   The above copyright notice and this permission notice shall be included in
   all copies or substantial portions of the Software.

   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
   THE AUTHOR(S) BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
   IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
   CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
**
********************************************************************************
**	This code was derived from the following source of information:
**
**	Matrox mga2064w reference internet edition
**	XFree86 3.3.1 mga driver
**
*/

#include <kgi/module.h>

#include <Matrox/mga_common_regs.h>
#include <Matrox/mga_common_mmio.h>
#include <Matrox/mga_common_text16.h>
#include <Matrox/mga_common.h>

/*
** VGA SEQuencer registers
*/
kgiu8 SEQ_in8(struct kgi_display *dpy, kgiu8 reg)
{
	GET_PRIVATE;
	kgiu8 val;

	mem_out8(reg, MMIO_SEQI);
	val = mem_in8(MMIO_SEQD);
	DEBUG3("SEQ mem %.2x -> %.2x", reg, val);
	return val;
}

void SEQ_out8(struct kgi_display *dpy, kgiu8 val, kgiu8 reg)
{
	GET_PRIVATE;
   
	DEBUG3("SEQ mem %.2x <- %.2x", reg, val);
	mem_out8(reg, MMIO_SEQI);
	mem_out8(val, MMIO_SEQD);
}

/*
** MISCellaneous output
*/
kgiu8 MISC_in8(struct kgi_display *dpy)
{
	GET_PRIVATE;
   
	register kgiu8 val = mem_in8(MMIO_MISCr);
	DEBUG3("MISC mem -> %.2x", val);
	return val;
}

void MISC_out8(struct kgi_display *dpy, kgiu8 val)
{
	GET_PRIVATE;
   
	mem_out8(val | MISC_COLOR_IO, MMIO_MISCw);
	DEBUG3("MISC mem <- %.2x", val | MISC_COLOR_IO);
}

/*
** VGA graphicx controller
*/
kgiu8 GRC_in8(struct kgi_display *dpy, kgiu8 reg)
{
	GET_PRIVATE;
	kgiu8 val;
   
	mem_out8(reg, MMIO_GRCI);
	val = mem_in8(MMIO_GRCD);
	DEBUG3("GRC mem %.2x -> %.2x", reg, val);
	return val;
}

void GRC_out8(struct kgi_display *dpy, kgiu8 val, kgiu8 reg)
{
	GET_PRIVATE;
   
	DEBUG3("GRC mem %.2x <- %.2x", reg, val);
	mem_out8(reg, MMIO_GRCI);
	mem_out8(val, MMIO_GRCD);
}

/* 
** Matrox' extended CRTC registers 
*/
kgiu8 ECRT_in8(struct kgi_display *dpy, kgiu8 reg)
{
	GET_PRIVATE;
   
	kgiu8 val;
 
	mem_out8(reg, MMIO_CRTXI);
	val = mem_in8(MMIO_CRTXD);
	DEBUG3("ECRT mem %.2x -> %.2x", reg, val);
	return val;
}

void ECRT_out8(struct kgi_display *dpy, kgiu8 val, kgiu8 reg)
{
	GET_PRIVATE;
   
	DEBUG3("ECRT mem %.2x <- %.2x", reg, val);
	mem_out16((val << 8) | reg, MMIO_CRTXI);
}

/*
** VGA CRT registers
*/
kgiu8 CRT_in8(struct kgi_display *dpy, kgiu8 reg)
{
	GET_PRIVATE;
   
	kgiu8 val;
 
	mem_out8(reg, MMIO_CRTI);
	val = mem_in8(MMIO_CRTD);
	DEBUG3("CRT mem %.2x -> %.2x", reg, val);
	return val;
}

void CRT_out8(struct kgi_display *dpy, kgiu8 val, kgiu8 reg)
{
	GET_PRIVATE;
   
	DEBUG3("CRT mem %.2x <- %.2x", reg, val);
	mem_out8(reg, MMIO_CRTI);
	mem_out8(val, MMIO_CRTD);
}

/*
**	Clock I/O functions
*/
void CLK_out8(struct kgi_display *dpy, kgiu8 value, kgiu8 reg)
{
	GET_PRIVATE;
   
	ASSERT(reg == 0);
	mem_out8((mem_in8(MMIO_MISCr) & ~MISC_CLOCK_MASK) | 
		((value << 2) & MISC_CLOCK_MASK), MMIO_MISCw);
}
#if 0
kgiu8 CLK_in8(struct kgi_display *dpy, kgiu8 reg)
{
	GET_PRIVATE;
   
	ASSERT(reg == 0);
	return (mem_in8(MMIO_MISCr) & MISC_CLOCK_MASK) >> 2;
}
#endif
/*
** Attribute controller
*/
#if 0
static kgiu8 ATC_in8(struct kgi_display *dpy, kgiu8 reg)
{
	GET_PRIVATE;
	kgiu8 val;

	mem_in8(MMIO_INS1r);	/* reset AFF and pas=0	*/
	mem_out8(reg, MMIO_ATCI);	/* set index	*/
	val = mem_in8(MMIO_ATCDr);	/* read value	*/
   
	mem_in8(MMIO_INS1r); /* reset AFF and pas=1, back to normal operation */
	mem_out8(ATCI_ENABLEDISPLAY, MMIO_ATCI);
	return val;
}
#endif
void ATC_out8(struct kgi_display *dpy, kgiu8 val, kgiu8 reg)
{
	GET_PRIVATE;
   
	mem_in8(MMIO_INS1r);	/* reset AFF and pas=0	*/
	mem_out8(reg, MMIO_ATCI);	/* set index	*/
	mem_out8(val, MMIO_ATCDw);	/* write value	*/
   
	mem_in8(MMIO_INS1r); /* reset AFF and pas=1, back to normal operation */
	mem_out8(ATCI_ENABLEDISPLAY, MMIO_ATCI);
}

void restore_ExtVga(struct kgi_display *dpy, kgiu8 *buf)
{
	GET_PRIVATE;
	register kgiu8 i;
   
	for (i = 0; i < NoExtVgaRegs; i++)
		mem_out16((*(buf++) << 8 ) | i, MMIO_CRTXI);
}

void restore_SEQ(struct kgi_display *dpy, kgiu8 *buf)
{
	register kgiu8 i;

	for (i = 0; i < NoSEQRegs; i++)
		SEQ_out8(dpy, *(buf++), i);
}

void restore_CRT(struct kgi_display *dpy, kgiu8 *buf)
{
	register kgiu8 i;
   
	CRT_out8(dpy, CRT_in8(dpy, CRT_VSYNCEND) & ~CR11_LOCKTIMING, CRT_VSYNCEND);
	CRT_out8(dpy, CRT_in8(dpy, CRT_HBLANKEND) | CR03_IS_VGA,CRT_HBLANKEND);
   
	for (i = 0; i < NoCRTRegs; i++) {

		switch (i) {

		case CRT_HBLANKEND:
			*buf |= CR03_IS_VGA;
			break;

		case CRT_VSYNCEND:
			*buf &= ~CR11_LOCKTIMING;
			break;

		default:
			break;
		}
		CRT_out8(dpy, *(buf++), i);
	}   
}

void restore_GRC(struct kgi_display *dpy, kgiu8 *buf)
{
	register kgiu8 i;

	for (i = 0; i < NoGRCRegs; i++)
		GRC_out8(dpy, *(buf++), i);
}

void restore_ATC(struct kgi_display *dpy, kgiu8 *buf)
{
	register kgiu8 i;

	for (i = 0; i < NoATCRegs; i++)
		ATC_out8(dpy, *(buf++), i);
}
